@@ -7,6 +7,8 @@ import warnings
 import matplotlib.pyplot as plt
 from matplotlib import cm
 from functools import wraps
+from numpy import AxisError
+import pickle
 
 
 __all__ = ('Dataset', 'Data',)
@@ -30,12 +32,15 @@ def construct_Dataset_from_dataframe(function):
 	return wrapper
 
 def _convert_ITP_to_path_to_index(index_to_path):
-	"""convert index_to_path pandas series to path_to_index dict 
-	----
-	index_to_path:(pandas.series)
+	"""Convert index_to_path (``pandas.Series``) to path_to_index (``dict``) 
+	
+
+	args:
+		index_to_path (pandas.Series): index_to_path to convert
+
+	returns:
+		(dict): path_to_index, key is path and value is list of indices for that path
 
-	returns
-	path_to_index:(dict) {path:[indices corresponding to that path]}
 	"""
 	path_to_index = dict()
 	for i in index_to_path.index:
@@ -52,8 +57,9 @@ def _check_file_exists(path, filename):
 	else:
 		return False
 
+
 def _remove_nans_from_set(set_to_remove_from):
-	"""Remove multiple nans from a set. Sometimes in Dataset.summary, it returns many nans when it should return only one. Quick fix for that issue"""
+	"""Remove multiple nans from a set."""
 
 	out = set()
 	for item in set_to_remove_from:
@@ -70,31 +76,43 @@ def _remove_nans_from_set(set_to_remove_from):
 	return out
 
 class Dataset(pd.DataFrame):
-	"""Dataset class for analysis. 
-
-	Dataset is a subclass of pandas.DataFrame. Used to manipulate meta data while keeping track of location for the real data, which can be retrieved when necessary.
+	"""Dataset class for analysis. Subclass of pandas.DataFrame. Used to manipulate meta data while keeping track of location for the real data, which can be retrieved when necessary.
 
 	Args:
-		path (str or dict): a path to where the real data lives. if dict, form is 
+		path (str or dict): Path to the real data. 
+		initializer (pandas.DataFrame or dict):  Initializer for a DataFrame. The meta data.
+		readfileby (function): How to read the data. Default is ``pandas.read_csv``
+		pointercolumn (str or index): Column name which holds name of file. Default is ``'filename'``
 
-				{path: [indices of initializer for this path]} 
+	Examples:
 
-		initializer (pandas.DataFrame):  the meta data. one column must contain a pointer (i.e. filename) to where the real data is stored
-		readfileby (function): how to read the data. default of None corresponds to 
+		.. code-block:: python
+			
+			>>> meta_data = pd.DataFrame(
+				{
+					'voltageApplied':['1v','2v'],
+					'filename':['t1.csv','t2.csv']
+				}
+			)
+
+			#Assuming the data is stored in './data/', create the Dataset
+			>>> dset = Dataset('./data/', meta_data,)
+
+			#Query the meta data for when voltageApplied is '1v':
+			>>> dset.query('voltageApplied == "1v"')
+
+			#return the Data
+			>>> dset.get_data()
 
-				pandas.read_csv()
 
 	"""
 
-	def __init__(self, path, initializer, readfileby=None):
+	def __init__(self, path, initializer, readfileby=pd.read_csv, pointercolumn = 'filename'):
 		super().__init__(initializer)
 		self.attrs['path'] = path
 		self.attrs['index_to_path'] = self._construct_index_to_path(path, initializer)
-		self.pointercolumn = 'filename'
-		if readfileby == None:
-			self.readfileby = lambda file: pd.read_csv(file)
-		else:
-			self.readfileby = readfileby
+		self.pointercolumn = pointercolumn
+		self.readfileby = readfileby
 
 	@property  
 	def _is_empty(self):
@@ -110,7 +128,13 @@ class Dataset(pd.DataFrame):
 
 	@property
 	def index_to_path(self):
-		"""Return Dict of index and corresponding path"""
+		"""Index to path ``pandas.Series``
+
+		returns:
+			(pandas.Series): Index to path
+
+
+		"""
 		return self.attrs['index_to_path']
 
 	@property
@@ -118,7 +142,40 @@ class Dataset(pd.DataFrame):
 		"""Return a brief summary of the data in your Dataset. 
 
 		Returns:
-			(Dict): a summary of the Dataset"""
+			(Dict): a summary of the Dataset. keys are columns names, values are sets of values appearing in the Dataset.
+
+		Examples:
+
+			.. code-block:: python
+
+				>>> dset.head()
+				identifier  pulsewidth_ns  delay_ns  high_voltage_v  preset_voltage_v  \
+				0      185um          100.0     200.0            0.25               0.5   
+				1      185um          100.0     200.0            0.25               0.5   
+				2      185um          100.0     200.0            0.25               0.5   
+				3      185um          100.0     200.0            0.25               0.5   
+				4      185um          100.0     200.0            0.25               0.5   
+
+				   preset_pulsewidth_ns                                      filename  trial  
+				0               10000.0  185um100e-9_200e-9_0x25V_500mv_10000ns_0.csv      0  
+				1               10000.0  185um100e-9_200e-9_0x25V_500mv_10000ns_1.csv      1  
+				2               10000.0  185um100e-9_200e-9_0x25V_500mv_10000ns_2.csv      2  
+				3               10000.0  185um100e-9_200e-9_0x25V_500mv_10000ns_3.csv      3  
+				4               10000.0  185um100e-9_200e-9_0x25V_500mv_10000ns_4.csv      4  
+				
+				>>> dset.summary
+				{
+					'identifier': {'185um'},
+					'pulsewidth_ns': {10.0, 50.0, 100.0},
+					'delay_ns': {10.0, 20.0, 50.0, 200.0, 300.0, 500.0},
+					'high_voltage_v': {0.125, 0.25, 0.375, 0.5, 0.75, 1.0, 1.5, 2.0, 2.5},
+					'preset_voltage_v': {0.5},
+					'preset_pulsewidth_ns': {10000.0},
+					'trial': {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}
+				}
+
+
+		"""
 		summary = dict()
 		for column in self.columns:
 			if column == self.pointercolumn:
@@ -145,6 +202,45 @@ class Dataset(pd.DataFrame):
 
 	@construct_Dataset_from_dataframe
 	def filter_on_column(self, column, function, **kwargs_for_function):
+		"""Filter Dataset. Keeps rows with column satisfying function.
+
+		args:
+			column (str or index): Specify column
+			function (function): Filter function. ``function(value) -> bool``
+			kwargs_for_function (kwargs): kwargs to pass to function
+
+		returns:
+			(Dataset): Filtered Dataset.
+
+		examples:
+
+			.. code-block:: python
+
+				>>> dset.summary
+				{
+					'identifier': {'185um'},
+					'pulsewidth_ns': {10.0, 50.0, 100.0},
+					'delay_ns': {10.0, 20.0, 50.0, 200.0, 300.0, 500.0},
+					'high_voltage_v': {0.125, 0.25, 0.375, 0.5, 0.75, 1.0, 1.5, 2.0, 2.5},
+					'preset_voltage_v': {0.5},
+					'preset_pulsewidth_ns': {10000.0},
+					'trial': {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}
+				}
+				
+				#Return only rows with where the high_voltage_v is greater than 1 and print the summary
+				>>> dset.filter_on_column('high_voltage_v', lambda x: x>1).summary
+				{	'identifier': {'185um'},
+					'pulsewidth_ns': {50.0},
+					'delay_ns': {10.0},
+					'high_voltage_v': {1.5, 2.0, 2.5},
+					'preset_voltage_v': {0.5},
+					'preset_pulsewidth_ns': {10000.0},
+					'trial': {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}
+				}
+
+
+
+		"""
 		return self[self[column].apply(function, **kwargs_for_function).values].reset_index(drop = True)
 
 	@construct_Dataset_from_dataframe
@@ -155,12 +251,23 @@ class Dataset(pd.DataFrame):
 			index (int or index): Index to select
 
 		returns:
-			(Dataset): Single row dataset.
+			(Dataset): Single row Dataset.
 
 
 		"""
 		return pd.DataFrame(self.iloc[index]).T.reset_index(drop = True)
 
+	def _write_ekpds_file(self, filename):
+		preamble = 'pointercolumn:{}|'.format(self.pointercolumn)
+		with open(filename, 'wb+') as f:
+			f.write(pickle.dumps(preamble))
+			f.write(b'########')
+			f.write(pickle.dumps(self.readfileby))
+			f.write(b'##|##|##|##')
+			f.write(pickle.dumps(self))
+			
+		return
+
 	def remove_index(self, index):
 		"""
 		Remove an index or array-like of indices.
@@ -182,7 +289,7 @@ class Dataset(pd.DataFrame):
 	
 	def remove_nonexistent_files_from_metadata(self):
 		"""
-		Remove references to files that do not exist in path.
+		Remove references to files that do not exist in path. This may occur, for example, if you know certain data files are bad (and thus delete them from the data dir), but did not delete them while collecting data. 
 		"""
 		remove_index = []
 		for ind, path in enumerate(self.index_to_path):
@@ -192,13 +299,14 @@ class Dataset(pd.DataFrame):
 				remove_index.append(ind)
 		
 		return self.remove_index(remove_index)
+
 	
 	def _construct_index_to_path(self, path, initializer):
 		"""Construct index_to_path from path provided
 		
 		args:
-			path (str or Dict): a path to where the real data lives. if dict, form is {path: [indices of initializer for this path]}  
-			initializer (pandas.DataFrame: meta data. one column must contain a pointer (filename) to where each the real data is stored
+			path (str or Dict): A path to where the real data lives. if dict, form is {path: [indices of initializer for this path]}  
+			initializer (pandas.DataFrame): Meta data. one column must contain a pointer (filename) to where each the real data is stored
 
 		"""
 		if len(self) == 0:
@@ -285,14 +393,17 @@ class Dataset(pd.DataFrame):
 			(Dataset): Updated Dataset.
 			
 		examples:
+			
 			Convert 25um and 10um to measured areas. This will only work if no other diameters are present in the Dataset.
-			```
-			>>>def how(dataframe):
-					nominal_diameter_to_measured_area_dict = {'25um':190, '10um':60}
-					return [nominal_diameter_to_measured_area_dict[x] for x in dataframe['diameter'].values]
-			>>>dset.add_calculated_column('measured_area_um', how = how)
-			```
-		
+			
+			.. code-block:: python
+
+				>>> def how(dataframe):
+						nominal_diameter_to_measured_area_dict = {'25um':190, '10um':60}
+						return [nominal_diameter_to_measured_area_dict[x] for x in dataframe['diameter'].values]
+
+				>>> dset.add_calculated_column('measured_area_um', how = how)
+
 		
 		"""
 		
@@ -303,6 +414,94 @@ class Dataset(pd.DataFrame):
 		self.add_column(column_name, new_column_data)
 		
 		return self
+
+	def save_meta_data(self,):
+		"""Save the current meta_data as ``pandas.DataFrame`` to path. This is not allowed for merged datasets *i.e.* Dataset resulting from ``analysis.utils.merge_Datasets``. To save a Dataset (including merged) see :func:`to_ekpds <.ekpds>`.
+		
+		"""
+		if len(set(self.index_to_path)) != 1:
+			raise ValueError('Dataset does not contain a unique path to data. Saving of this type of Dataset is not supported.')
+
+		path = list(set(self.index_to_path))[0]
+		pd.DataFrame(self).to_pickle(path + 'meta_data')
+
+		return
+
+	def to_ekpds(self, path):
+		"""Save Dataset to file (extension .ekpds).
+
+		args:
+			path (str): Path to save location
+
+		example:
+
+			.. code-block:: python
+				
+				dset.to_ekpds('./dset1.ekpds')
+
+		"""
+		self._set_index_to_path_absolute()
+		self._set_path_absolute()
+
+		#check if file exits:
+		dirname = os.path.dirname(path)
+		name = path.split('/')[-1]
+		if _check_file_exists(dirname, name):
+			yn = input('file ({}) already exists. Overwrite? (y/n)'.format(path))
+			if yn.lower() == 'y':
+				os.remove(path)
+				self._write_ekpds_file(path)
+			else:
+				pass
+		else:
+			self._write_ekpds_file(path)
+
+		return 
+
+
+
+
+	def _set_index_to_path_absolute(self):
+		"""Modify ``.index_to_path`` to absolute paths."""
+		itp = self.index_to_path
+
+		for ind, x in zip(itp.index, itp):
+			abspath = (os.path.abspath(x)).replace('\\', '/')
+			if not abspath[-1] == '/':
+				abspath += '/' #put final slash in place
+				
+			itp.at[ind] = abspath
+
+		return
+
+	def _set_path_absolute(self):
+		"""Modify ``.path`` to absolute paths."""
+		if type(self.path) == str:
+			abspath = (os.path.abspath(self.path)).replace('\\', '/')
+			if not abspath[-1] == '/':
+				abspath += '/' #put final slash in place
+			out = abspath
+		elif type(self.path) == dict:
+			pdict = self.path
+			new_pdict = {}
+			for key in pdict:
+				indices = pdict[key]
+				
+				abspath = (os.path.abspath(key)).replace('\\', '/')
+				if not abspath[-1] == '/':
+					abspath += '/' #put final slash in place
+				
+				new_pdict.update({abspath:indices})
+			del pdict
+			out = new_pdict
+		else:
+			raise TypeError('Only str or dict supported as self.path. Please report this issue.')
+
+		self.attrs['path'] = out
+
+		return
+
+
 	
 	def add_column(self, column_name, column_data):
 		"""Add a column to a Dataset.
@@ -318,7 +517,7 @@ class Dataset(pd.DataFrame):
 
 	def get_data(self, groupby=None, labelby=None,):
 		"""
-		Return data in Data (Data class) for the current Dataset: returns Data class 
+		Return data in Data (Data class) for the current Dataset. If using groupby kwarg, resulting Data will vstack all data which corresponds to that grouping. (See examples)
 		
 		args:
 				groupby (str, label, index or array-like of):  what to group on
@@ -326,6 +525,67 @@ class Dataset(pd.DataFrame):
 
 		returns:
 				(Data): the data
+
+		examples:
+			
+			.. code-block:: python
+
+				>>> dset.summary
+				{
+					'identifier': {'185um'},
+					'pulsewidth_ns': {10.0, 50.0, 100.0},
+					'delay_ns': {10.0, 20.0, 50.0, 200.0, 300.0, 500.0},
+					'high_voltage_v': {0.125, 0.25, 0.375, 0.5, 0.75, 1.0, 1.5, 2.0, 2.5},
+					'preset_voltage_v': {0.5},
+					'preset_pulsewidth_ns': {10000.0},
+					'trial': {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}
+				}
+
+				>>> data = dset.get_data() #no groupby yet
+				>>> data.data_keys
+				['time', 'p1', 'p2']
+
+				>>> data[0]['data']['p1'] # This corresponds to a single trial (1D data)
+				array([  0.00898495,  0.00765674,  0.00351585, ..., -0.00679731,
+						-0.00101569, -0.00039065])
+
+				# now we will group by high_voltage_v. There are many .csv files that correspond to such a case
+				# for example how many are there for a high_voltage_v of .125? (there are 5, see here)
+				>>> len(dset.query('high_voltage_v == .125'))
+				5
+
+				# let's retrieve the data but with grouping
+				>>> data = dset.get_data(groupby = 'high_voltage_v')
+				>>> data[0]['data']['p1'] #vstack of all different .csv files grouped by high_voltage_v (a meta data parameter)
+				array([[ 0.00898495,  0.00765674,  0.00351585, ..., -0.00679731,
+						-0.00101569, -0.00039065],
+					   [-0.02172014, -0.02773615, -0.03695549, ..., -0.0203138 ,
+						-0.0085943 , -0.00117195],
+					   [-0.0351585 , -0.02859558, -0.02289209, ..., -0.03093948,
+						-0.01968876, -0.01289145],
+					   [-0.02765802, -0.02453282, -0.02070445, ..., -0.00679731,
+						-0.00257829, -0.0093756 ],
+					   [-0.0375024 , -0.04656548, -0.04930003, ..., -0.03789305,
+						-0.02609542, -0.01632917]])
+
+				# no longer is it 1D data. 
+				>>> data[0]['data']['p1'].shape
+				(5, 500)
+
+				#recall that there were 5 rows in the Dataset corresponding to a high_voltage_v of .125. Is this that grouping? we can check:
+				>>> data[0]['definition']
+				# indeed it is!!!
+				{	
+					'identifier': {'185um'},
+					'pulsewidth_ns': {10.0},
+					'delay_ns': {20.0},
+					'high_voltage_v': {0.125},
+					'preset_voltage_v': {0.5},
+					'preset_pulsewidth_ns': {10000.0},
+					'trial': {0, 1, 2, 3, 4}
+				}
+
+
 		"""
 		if len(self) == 0:
 			raise ValueError('No meta data to return data for!!')
@@ -454,47 +714,39 @@ class iDataIndexer():
 		return Data({index: self.indexed_dict[index]})
 
 class Data(dict):
-	"""Data class for maintaining and manipulating the real data. Typically retrieved via Dataset.get_data()
+	"""Data class for maintaining and manipulating the real data. Typically retrieved via ``Dataset.get_data()``
+
 
 	args:
 		dict (Dict): a dict (with form shown below) of the data. 
 
 
-	The real data. Subclass of dict. Each piece of data (may be grouped) is given a numerical index and structure as follows: 
+	Examples:
 
-	```
-	{index:
-		{
-		'data':dict({'key':np.array(the real data)}), 
-		'definition':dict(describing which data is contained)
-		}
-	}
-	```
-	for example, an instance of the Data class corresponding to an experiment may be:
-	
-	```
-	{0: {
-		'definition': {'frequency': {'1067hz'},
-					   'amplitude': {'500ua'},
-					   'nave': {5},
-					   'low_current': {-2.5},
-					   'high_current': {2.5},
-					   'delay': {1},
-					   'time_constant': {'30ms'},
-					   'ramp_rate': {0.05},
-					   'ramp_up_first': {True},
-					   'identifier': {'B1'},
-					   'sensitivity': {'50mv/na'},
-					   'trial': {0}},
-		'data': {'H_mean': array([-403.524  , -407.18   , -395.602  , -382.146  , -367.444  , ... ]),
-			   'H_std': array([10.39320663,  4.7261697 ,  5.3691951 ,  5.71281577,  5.9829578 , ... ]),
-			   'R_mean': array([0.0324881 , 0.03248582, 0.03248772, 0.03248544, 0.03248354, ... ]),
-			   'R_std': array([1.69941166e-06, 1.42182981e-06, 1.42182981e-06, 3.31276320e-06, ...]),
-			   'Theta_mean': array([0.0324881 , 0.03248582, 0.03248772, 0.03248544, 0.03248354, ...]),
-			   'Theta_std': array([0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , ...])}
-		}
-	}
-	```
+		.. code-block:: python
+
+			>>> data
+			> {0: {
+				'definition': {'frequency': {'1067hz'},
+							   'amplitude': {'500ua'},
+							   'nave': {5},
+							   'low_current': {-2.5},
+							   'high_current': {2.5},
+							   'delay': {1},
+							   'time_constant': {'30ms'},
+							   'ramp_rate': {0.05},
+							   'ramp_up_first': {True},
+							   'identifier': {'B1'},
+							   'sensitivity': {'50mv/na'},
+							   'trial': {0}},
+				'data': {'H_mean': array([-403.524  , -407.18   , -395.602  , -382.146  , -367.444  , ... ]),
+					   'H_std': array([10.39320663,  4.7261697 ,  5.3691951 ,  5.71281577,  5.9829578 , ... ]),
+					   'R_mean': array([0.0324881 , 0.03248582, 0.03248772, 0.03248544, 0.03248354, ... ]),
+					   'R_std': array([1.69941166e-06, 1.42182981e-06, 1.42182981e-06, 3.31276320e-06, ...]),
+					   'Theta_mean': array([0.0324881 , 0.03248582, 0.03248772, 0.03248544, 0.03248354, ...]),
+					   'Theta_std': array([0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , 0.    , ...])}
+				}
+			}
 
 
 	"""
@@ -512,9 +764,10 @@ class Data(dict):
 
 		example:
 
-			```
-			>>> Data.iloc[0]
-			```
+			.. code-block:: python
+				
+				>>> Data.iloc[0]
+			
 		"""
 		return iDataIndexer(self.to_dict().copy())
 
@@ -536,7 +789,7 @@ class Data(dict):
 		returns:
 			(list): data keys
 		"""
-		return list(self[0]['data'].keys())
+		return list(self[list(self.keys())[0]]['data'].keys())
 
 	def _get_indices_satisfying_definition_condtion(self, condition):
 		"""need docstring"""
@@ -567,22 +820,30 @@ class Data(dict):
 		returns:
 			(Data): filtered Data
 
-		Example:
+		Examples:
 
-		Filter the data such that the data key 'R' only contains values >10. This leaves all other data keys unchanged.
-		```
-		>>> Data.filter({'R': lambda x: x>10}) 
-		```
-		Filter data corresponding to an amplitude of '500ua' such that data key 'R' contains only values >10.
-		```
-		>>> Data.filter({'R': lambda x: x>10}, {'amplitude':'500ua'}) 
-		```
-		Filter data based on 'saturation' but also filter the 'switching_time' data.
-		```
-		>>> Data.filter({'saturation': lambda x: x > .003}, additional_data_keys_to_filter='switching_time')
-		``` 
+			Filter the data such that the data key 'R' only contains values >10. This leaves all other data keys unchanged.
+			
+			.. code-block:: python
 
-		You may use this method to remove outliers, for example.
+				>>> Data.filter({'R': lambda x: x>10}) 
+			
+
+
+			Filter data corresponding to an amplitude of '500ua' such that data key 'R' contains only values >10.
+			
+			.. code-block:: python
+
+				>>> Data.filter({'R': lambda x: x>10}, {'amplitude':'500ua'}) 
+	
+			Filter data based on 'saturation' but also filter the 'switching_time' data.
+		
+			.. code-block:: python
+
+				>>> Data.filter({'saturation': lambda x: x > .003}, additional_data_keys_to_filter='switching_time')
+	
+
+			You may use this method to remove outliers, for example.
 
 		"""
 		if definition_condition_dict == None:
@@ -633,19 +894,19 @@ class Data(dict):
 		returns:
 			(Data): data satisfying condition 
 		
-		example:
+		examples:
+
+			.. code-block:: python
+				
+				#Providing multiple values will be joined by logical or, i.e.
+				Data.contains({'high_voltage_v':{'100mv', '200mv'}})
+				#will search for all data with '100mv' OR '200mv' for high_voltage_v. 
+
+				#Multiple keys provided will be joined with logical AND. i.e. 
+				Data.contains({'x':1, 'y':2})
+				#will search for x = 1 AND y = 2.
 
-		Providing multiple values will be joined by logical or, i.e.
-		```
-		Data.contains({'high_voltage_v':{'100mv', '200mv'}})
-		```
-		will search for all data with '100mv' OR '200mv' for high_voltage_v. 
 
-		Multiple keys provided will be joined with logical AND. i.e. 
-		```
-		Data.contains({'x':1, 'y':2})
-		``` 
-		will search for x = 1 AND y = 2.
 	
 		"""
 		indices = self._get_indices_satisfying_definition_condtion(condition)
@@ -691,61 +952,64 @@ class Data(dict):
 					if len(data[k].shape) == 1: #1d data
 						mean_data.update({k:data[k]})
 					else:
-						mean_data.update({k:np.mean(data[k], axis = 0)})
+						try:
+							mean_data.update({k:np.mean(data[k], axis = 0)})
+						except AxisError:
+							mean_data.update({k:data[k]})
 				tmp_out[key].update({'data':mean_data})
 			return Data(tmp_out)
 
 	def apply(self, function_on_data, pass_defn = False, kwargs_for_function=None,**kwargs):
-		"""Apply data_function to the data in each index. **kwargs will be passed to data_function.
+		"""Apply data_function to the data in each index. ``**kwargs`` will be passed to data_function.
 
 		args:
 			function_on_data (function): f(dict) -> dict. Function is passed the data_dict (corresponding to self[index]['data']).
 			pass_defn (bool): Whether or not to pass the definition to function_on_data. If True, will be passed with other kwargs. 
-			kwargs (**kwargs): Additional arguments for function_on_data. 
 			kwargs_for_function (dict): Carryover from an older version. This serves as a error catch to help users convert older code. This argument is not supported.
 
 		returns:
 				(Data): the new data after operating on it
 
-		example:
+		examples:
 
-		```
-		>>> some_data
-		>
-		{
-			0: {'definition': {'param1': {'10V'},
-				'param2': {'100ns', '10ns', '50ns'},
-				'param3': {'1mv', '2mv'}},
-				'data': {'raw_data': array([[1, 2, 3],
-					  [1, 2, 3],
-					  [1, 2, 3],
-					  [1, 2, 3]], dtype=int64)}},
-			1: {'definition': {'param1': {'5V'}, 'param2': {'100ns'}, 'param3': {'1mv'}},
-				'data': {'raw_data': array([1, 2, 3], dtype=int64)}}
-		}
+			.. code-block:: python
+				
+				>>> some_data
+				>
+				{
+					0: {'definition': {'param1': {'10V'},
+						'param2': {'100ns', '10ns', '50ns'},
+						'param3': {'1mv', '2mv'}},
+						'data': {'raw_data': array([[1, 2, 3],
+							  [1, 2, 3],
+							  [1, 2, 3],
+							  [1, 2, 3]], dtype=int64)}},
+					1: {'definition': {'param1': {'5V'}, 'param2': {'100ns'}, 'param3': {'1mv'}},
+						'data': {'raw_data': array([1, 2, 3], dtype=int64)}}
+				}
 
-		#some function will square the data
+				#some function will square the data
 
-		def some_function(data_dict):
-			"a function which operates on the data dict and returns a data dict"
-			out = dict()
-			for key in data_dict:
-				out.update({key:data_dict[key]**2})
-			return out
+				def some_function(data_dict):
+					"a function which operates on the data dict and returns a data dict"
+					out = dict()
+					for key in data_dict:
+						out.update({key:data_dict[key]**2})
+					return out
+
+				>>> some_data.apply(some_function)
+				>
+				{
+					0: {'definition': {'param1': {'10V'},
+						'param2': {'100ns', '10ns', '50ns'},
+						'param3': {'1mv', '2mv'}},
+						'data': {'raw_data': array([[1, 4, 9],
+						  [1, 4, 9],
+						  [1, 4, 9],
+						  [1, 4, 9]], dtype=int64)}},
+					1: {'definition': {'param1': {'5V'}, 'param2': {'100ns'}, 'param3': {'1mv'}},
+						'data': {'raw_data': array([1, 4, 9], dtype=int64)}}}
 
-		>>> some_data.apply(some_function)
-		>
-		{
-			0: {'definition': {'param1': {'10V'},
-				'param2': {'100ns', '10ns', '50ns'},
-				'param3': {'1mv', '2mv'}},
-				'data': {'raw_data': array([[1, 4, 9],
-				  [1, 4, 9],
-				  [1, 4, 9],
-				  [1, 4, 9]], dtype=int64)}},
-			1: {'definition': {'param1': {'5V'}, 'param2': {'100ns'}, 'param3': {'1mv'}},
-				'data': {'raw_data': array([1, 4, 9], dtype=int64)}}}
-		```
 		"""
 		if type(kwargs_for_function) != type(None):
 			raise ValueError("kwargs_for_function argument is no longer supported as of version 0.0.7. Pass kwargs for the apply function simply as kwargs in .apply()")
