@@ -4,6 +4,8 @@ import time
 import itertools
 import warnings
 import pyvisa
+import os
+from IPython import display
 
 from .misc import get_save_name
 
@@ -45,7 +47,49 @@ class experiment():
 		print(self.run_function.__code__.co_varnames)
 		return
 
-	def config_path(self, path):
+	def config_path(self, path, directory_delimiter = '/'):
+		"""Config the path to save data. If path does not exist, user will be prompted to create. 
+
+		args:
+			path (str): Path
+			directory_delimiter (str): Delimiter between directories in path. Sometimes it may be '\\' for Windows OS, for example.
+
+		"""
+		if path[-1] != directory_delimiter:
+			raise ValueError('must specify a directory. "{}" does not specify a directory'.format(path))
+
+		success = False
+		original_path = path
+
+		try:
+			os.listdir(path)
+			yn = 'n'
+		except FileNotFoundError:
+			yn = input("Path '{}' does not exist. Do you wish to create it? (y/n)".format(path))
+			yn = yn.lower()
+
+		if yn == 'y':
+			to_create = []
+
+			while success == False:
+				try:
+					os.listdir(path)
+					success = True
+				except FileNotFoundError:
+					path_spl = path.split(directory_delimiter)
+					to_create.append(path_spl[-2])
+					path = ''
+					for spl in path_spl[:-2]:
+						path+=spl + '/'
+
+			to_create = to_create[::-1]
+			for directory in to_create:
+				os.mkdir(path + directory + directory_delimiter)
+				path += directory + directory_delimiter
+				print('creating dir "{}"'.format(path))
+				
+		else:
+			pass
 		self.path = path
 
 	def checks(self, params):
@@ -63,7 +107,7 @@ class experiment():
 
 
 		Examples:
-			```
+			
 			>>> exp = experiment()
 			>>>	kw_scan_params = {
 					'frequency':['147hz', '47hz'],
@@ -88,7 +132,9 @@ class experiment():
 			>>> order = ['harmonic', 'frequency', 'amplitude']
 			#Perform scan
 			>>> exp.n_param_scan(kw_scan_params, fixed_params, order)
-			```
+			
+
+
 		"""
 		#check to make sure no errors
 		params = kw_scan_params.copy()
@@ -105,6 +151,8 @@ class experiment():
 		if set(scan_param_order) != set(kw_scan_params.keys()):
 			raise KeyError('kw_scan_params do not have the same keys as scan_param_order')
 		try:
+			total_scans = int(np.prod([len(kw_scan_params[x]) for x in kw_scan_params])*ntrials)
+			iteration = 0
 			for key in kw_scan_params:
 				params = kw_scan_params[key]
 				if hasattr(params, '__getitem__') and type(params) != str:
@@ -123,6 +171,9 @@ class experiment():
 				for i, key in enumerate(scan_param_order[::-1]):
 					kwargs.update({key:params[i]})
 				for count in range(ntrials):
+					iteration += 1
+					print('Scan {} of {}'.format(iteration, total_scans))
+					display.clear_output(wait=True)
 					trial(self.run_function, kwargs, self.path)
 					time.sleep(1)
 
